"""File generated by TLObjects' generator. All changes will be ERASED"""
from ...tl.tlobject import TLObject
from ...tl.tlobject import TLRequest
from typing import Optional, List, Union, TYPE_CHECKING
import os
import struct
from datetime import datetime


class DummyFunctionRequest(TLRequest):
    CONSTRUCTOR_ID = 0x56137993
    SUBCLASS_OF_ID = 0x53299ae6

    def to_dict(self):
        return {
            '_': 'DummyFunctionRequest'
        }

    def __bytes__(self):
        return b''.join((
            b'\x93y\x13V',
        ))

    @classmethod
    def from_reader(cls, reader):
        return cls()

