"""File generated by TLObjects' generator. All changes will be ERASED"""
from ...tl.tlobject import TLObject
from typing import Optional, List, Union, TYPE_CHECKING
import os
import struct
from datetime import datetime


class LiteResponse(TLObject):
    CONSTRUCTOR_ID = 0x764386d7
    SUBCLASS_OF_ID = 0x69479c1

    def __init__(self, response: bytes):
        """
        Constructor for wallet.LiteResponse: Instance of LiteResponse.
        """
        self.response = response

    def to_dict(self):
        return {
            '_': 'LiteResponse',
            'response': self.response
        }

    def __bytes__(self):
        return b''.join((
            b'\xd7\x86Cv',
            self.serialize_bytes(self.response),
        ))

    @classmethod
    def from_reader(cls, reader):
        _response = reader.tgread_bytes()
        return cls(response=_response)


class SecretSalt(TLObject):
    CONSTRUCTOR_ID = 0xdd484d64
    SUBCLASS_OF_ID = 0x1d4dbeee

    def __init__(self, salt: bytes):
        """
        Constructor for wallet.KeySecretSalt: Instance of SecretSalt.
        """
        self.salt = salt

    def to_dict(self):
        return {
            '_': 'SecretSalt',
            'salt': self.salt
        }

    def __bytes__(self):
        return b''.join((
            b'dMH\xdd',
            self.serialize_bytes(self.salt),
        ))

    @classmethod
    def from_reader(cls, reader):
        _salt = reader.tgread_bytes()
        return cls(salt=_salt)

