"""File generated by TLObjects' generator. All changes will be ERASED"""
from ...tl.tlobject import TLObject
from ...tl.tlobject import TLRequest
from typing import Optional, List, Union, TYPE_CHECKING
import os
import struct
from datetime import datetime


class GetKeySecretSaltRequest(TLRequest):
    CONSTRUCTOR_ID = 0xb57f346
    SUBCLASS_OF_ID = 0x1d4dbeee

    def __init__(self, revoke: bool):
        """
        :returns wallet.KeySecretSalt: Instance of SecretSalt.
        """
        self.revoke = revoke

    def to_dict(self):
        return {
            '_': 'GetKeySecretSaltRequest',
            'revoke': self.revoke
        }

    def __bytes__(self):
        return b''.join((
            b'F\xf3W\x0b',
            b'\xb5ur\x99' if self.revoke else b'7\x97y\xbc',
        ))

    @classmethod
    def from_reader(cls, reader):
        _revoke = reader.tgread_bool()
        return cls(revoke=_revoke)


class SendLiteRequestRequest(TLRequest):
    CONSTRUCTOR_ID = 0xe2c9d33e
    SUBCLASS_OF_ID = 0x69479c1

    def __init__(self, body: bytes):
        """
        :returns wallet.LiteResponse: Instance of LiteResponse.
        """
        self.body = body

    def to_dict(self):
        return {
            '_': 'SendLiteRequestRequest',
            'body': self.body
        }

    def __bytes__(self):
        return b''.join((
            b'>\xd3\xc9\xe2',
            self.serialize_bytes(self.body),
        ))

    @classmethod
    def from_reader(cls, reader):
        _body = reader.tgread_bytes()
        return cls(body=_body)

